/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.convertor;

import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import com.ibm.productivity.tools.ui.ProductivityToolsUtil;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.bridge.BridgeExistsException;
import com.sun.star.bridge.XBridge;
import com.sun.star.bridge.XBridgeFactory;
import com.sun.star.connection.XConnection;
import com.sun.star.container.XNameAccess;
import com.sun.star.document.XDocumentInfo;
import com.sun.star.document.XDocumentInfoSupplier;
import com.sun.star.document.XExporter;
import com.sun.star.document.XFilter;
import com.sun.star.drawing.XDrawPage;
import com.sun.star.drawing.XDrawPages;
import com.sun.star.drawing.XDrawPagesSupplier;
import com.sun.star.frame.FrameSearchFlag;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XStorable;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

public class DemoView extends ViewPart {

	private Button loadButton;

	private Button exportButton;

	private XComponent xDocument;

	private String sourceURL;

	private String filter;

	private boolean isExpToHtml = false;

	private boolean isExpToGraphic = false;

	private Text authorText;

	public DemoView() {
		super();
	}

	public void createPartControl(Composite composite) {
		composite.setLayout(new GridLayout(1, false));

		createLoadGroup(composite);

		createExportGroup(composite);

		createMetaGroup(composite);
	}

	private void createLoadGroup(Composite parent) {
		Group loadGroup = new Group(parent, SWT.FILL);

		loadGroup.setText("Implicitly Load");
		GridData data = new GridData(GridData.CENTER);
		data.grabExcessHorizontalSpace = true;
		loadGroup.setLayoutData(data);

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		loadGroup.setLayout(layout);

		Label loadDescription = new Label(loadGroup, SWT.NULL);
		loadDescription.setText("Loading implicitly means\n"
				+ "load a document of three\n" + "kind of Symphony into IBM\n"
				+ "Lotus Symphony but does\n" + "not show it up on Symphony");

		loadButton = new Button(loadGroup, SWT.PUSH | SWT.CENTER);
		loadButton.setText("Load Document implicitly");
		final Shell she = this.getViewSite().getShell();
		loadButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent event) {
				String filePath = new FileDialog(she).open();
				if (filePath != null)
					loadDocumentImplictly(filePath);
			}

		});
	}

	private void createExportGroup(Composite parent) {
		Group exportGroup = new Group(parent, SWT.FILL);

		exportGroup.setText("Export");
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		exportGroup.setLayoutData(data);

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		exportGroup.setLayout(layout);

		Label exportDescription = new Label(exportGroup, SWT.NULL);
		exportDescription.setText("Convert loaded document\n"
				+ "into HTML file or JEPG\n" + "image according its type,\n"
				+ "ODT and ODS into HTML\n" + "file, ODP into a array\n"
				+ "of JEPG images");

		exportButton = new Button(exportGroup, SWT.PUSH | SWT.CENTER);
		exportButton.setText("Export Loaded Document");
		exportButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent event) {
				resolveDocument();
				if (isExpToHtml)
					exportToHtml();
				if (isExpToGraphic)
					exportToGraphic();
			}
		});
	}

	private void createMetaGroup(Composite parent) {
		Group metaGroup = new Group(parent, SWT.FILL);
		metaGroup.setText("Meta information");
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		metaGroup.setLayoutData(data);

		metaGroup.setLayout(new GridLayout(2, false));
		Label authorLabel = new Label(metaGroup, SWT.LEFT);
		authorLabel.setText("Modified By:");
		authorText = new Text(metaGroup, SWT.BORDER);

		Button setAuthorButton = new Button(metaGroup, SWT.PUSH);
		setAuthorButton.setText("Set Value");
		setAuthorButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {
				XDocumentInfo info = getDocumentInfo();
				XPropertySet propSet = (XPropertySet) UnoRuntime
						.queryInterface(XPropertySet.class, info);
				try {
					propSet.setPropertyValue("ModifiedBy", authorText
									.getText());
					/*XStorable xStore = (XStorable) UnoRuntime.queryInterface(XStorable.class, xDocument);
					xStore.store();*/
				} catch (UnknownPropertyException e) {
					e.printStackTrace();
				} catch (WrappedTargetException e) {
					e.printStackTrace();
				} catch (PropertyVetoException e) {
					e.printStackTrace();
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				}
			}

		});

		Button getAuthorButton = new Button(metaGroup, SWT.PUSH);
		getAuthorButton.setText("Get Value");
		getAuthorButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {
				XDocumentInfo info = getDocumentInfo();
				XPropertySet propSet = (XPropertySet) UnoRuntime
						.queryInterface(XPropertySet.class, info);
				try {
					String author = (String) propSet
							.getPropertyValue("ModifiedBy");
					authorText.setText(author);
				} catch (UnknownPropertyException e) {
					e.printStackTrace();
				} catch (WrappedTargetException e) {
					e.printStackTrace();
				}
			}

		});
	}

	protected XDocumentInfo getDocumentInfo() {
		XDocumentInfoSupplier xSupplier = (XDocumentInfoSupplier) UnoRuntime
				.queryInterface(XDocumentInfoSupplier.class, xDocument);
		return xSupplier.getDocumentInfo();
	}

	protected void exportToGraphic() {
		try {
			XMultiServiceFactory xServiceFactory = getServiceFactory();
			Object GraphicExportFilter = xServiceFactory
					.createInstance("com.sun.star.drawing.GraphicExportFilter");
			XExporter xExporter = (XExporter) UnoRuntime.queryInterface(
					XExporter.class, GraphicExportFilter);
			PropertyValue aProps[] = new PropertyValue[2];
			aProps[0] = new PropertyValue();
			aProps[0].Name = "MediaType";
			aProps[0].Value = "image/jpeg";
			/**
			 * some graphics e.g. the Windows Metafile does not have a Media
			 * Type, for this case*aProps[0].Name = "FilterName"; it ispossible
			 * to set a FilterName aProps[0].Value ="WMF";
			 */
			int pageNum = getDrawPageCount(xDocument);
			String destBase = sourceURL
					.substring(0, sourceURL.lastIndexOf('.'));
			for (int i = 0; i < pageNum; i++) {
				String filePath = destBase + "Slide_" + i + ".jpg";
				aProps[1] = new PropertyValue();
				aProps[1].Name = "URL";
				aProps[1].Value = filePath;

				XDrawPage xPage = getDrawPageByIndex(xDocument, i);

				XComponent xComp = (XComponent) UnoRuntime.queryInterface(
						XComponent.class, xPage);
				xExporter.setSourceDocument(xComp);
				XFilter xFilter = (XFilter) UnoRuntime.queryInterface(
						XFilter.class, xExporter);
				xFilter.filter(aProps);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static XDrawPage getDrawPageByIndex(XComponent xComponent, int nIndex)
			throws com.sun.star.lang.IndexOutOfBoundsException,
			com.sun.star.lang.WrappedTargetException {

		XDrawPagesSupplier xDrawPagesSupplier = (XDrawPagesSupplier) UnoRuntime
				.queryInterface(XDrawPagesSupplier.class, xComponent);
		XDrawPages xDrawPages = xDrawPagesSupplier.getDrawPages();
		return (XDrawPage) UnoRuntime.queryInterface(XDrawPage.class,
				xDrawPages.getByIndex(nIndex));
	}

	private int getDrawPageCount(XComponent document) {
		XDrawPagesSupplier xDrawPagesSupplier = (XDrawPagesSupplier) UnoRuntime
				.queryInterface(XDrawPagesSupplier.class, document);
		XDrawPages xDrawPages = xDrawPagesSupplier.getDrawPages();
		return xDrawPages.getCount();
	}

	protected void resolveDocument() {
		XServiceInfo xInfo = (XServiceInfo) UnoRuntime.queryInterface(
				XServiceInfo.class, xDocument);
		if (xInfo != null) {
			if (xInfo.supportsService("com.sun.star.text.TextDocument")) {
				filter = new String("HTML (StarWriter)");
				isExpToHtml = true;
				isExpToGraphic = false;
			} else if (xInfo.supportsService("com.sun.star.text.WebDocument")) {
				filter = new String("HTML");
				isExpToHtml = true;
				isExpToGraphic = false;
			} else if (xInfo
					.supportsService("com.sun.star.sheet.SpreadsheetDocument")) {
				filter = new String("HTML (StarCalc)");
				isExpToHtml = true;
				isExpToGraphic = false;
			} else if (xInfo
					.supportsService("com.sun.star.presentation.PresentationDocument")) {
				isExpToHtml = false;
				isExpToGraphic = true;
			}
		}

	}

	protected void exportToHtml() {
		try {
			// Check for existing state of this filter.
			if (filter != null) {
				XMultiServiceFactory xSMGR = getServiceFactory();
				XNameAccess xFilterContainer = (XNameAccess) UnoRuntime
						.queryInterface(
								XNameAccess.class,
								xSMGR
										.createInstance("com.sun.star.document.FilterFactory"));
				if (xFilterContainer.hasByName(filter) == false)
					filter = null;
			}
			// Use this filter for export.
			if (filter != null) {
				PropertyValue[] lProperties = new PropertyValue[2];
				lProperties[0] = new PropertyValue();
				lProperties[0].Name = "FilterName";
				lProperties[0].Value = filter;
				lProperties[1] = new PropertyValue();
				lProperties[1].Name = "Overwrite";
				lProperties[1].Value = Boolean.TRUE;
				XStorable xStore = (XStorable) UnoRuntime.queryInterface(
						XStorable.class, xDocument);
				String destURL = sourceURL.replaceFirst("\\.[^.]+$", ".html");
				xStore.storeAsURL(destURL, lProperties);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	protected void loadDocumentImplictly(String filePath) {
		XMultiServiceFactory xServiceFactory = getServiceFactory();
		try {
			Object object = xServiceFactory
					.createInstance("com.sun.star.frame.Desktop");
			XComponentLoader loader = (XComponentLoader) UnoRuntime
					.queryInterface(XComponentLoader.class, object);
			PropertyValue[] aArgs = new PropertyValue[1];
			aArgs[0] = new PropertyValue();
			aArgs[0].Name = "Hidden";
			aArgs[0].Value = new Boolean(true);
			sourceURL = "file:///"
					+ Path.fromOSString(filePath).toPortableString();
			object = loader.loadComponentFromURL(sourceURL, "_blank",
					FrameSearchFlag.CREATE, aArgs);
			xDocument = (XComponent) UnoRuntime.queryInterface(
					XComponent.class, object);
			authorText.setText("");
		} catch (com.sun.star.io.IOException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static XMultiServiceFactory getServiceFactory() {
		XConnection conn = ProductivityToolsUtil.getUNOConnection();
		XBridge mBridge;
		try {
			XComponentContext _ctx = com.sun.star.comp.helper.Bootstrap
					.createInitialComponentContext(null);
			Object x = _ctx.getServiceManager().createInstanceWithContext(
					"com.sun.star.bridge.BridgeFactory", _ctx);
			XBridgeFactory xBridgeFactory = (XBridgeFactory) UnoRuntime
					.queryInterface(XBridgeFactory.class, x);
			// create a nameless bridge with no instance provider
			try {
				mBridge = xBridgeFactory.createBridge("SODC_Bridge", "urp",
						conn, null);
			} catch (BridgeExistsException beexp) {
				mBridge = xBridgeFactory.getBridge("SODC_Bridge");
			}
			// get the remote instance
			x = mBridge.getInstance("StarOffice.ServiceManager");
			// Did the remote server export this object?
			if (null == x)
				return null;
			// Query the initial object for its main factory interface
			XMultiComponentFactory xOfficeMultiComponentFactory = (XMultiComponentFactory) UnoRuntime
					.queryInterface(XMultiComponentFactory.class, x);
			// Retrieve the component context
			// Query on the XPropertySet interface.
			XPropertySet xProperySet = (XPropertySet) UnoRuntime
					.queryInterface(XPropertySet.class,
							xOfficeMultiComponentFactory);
			// Get the default context from the editor service.
			Object oDefaultContext = null;
			try {
				oDefaultContext = xProperySet
						.getPropertyValue("DefaultContext");
			} catch (UnknownPropertyException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			}
			if (oDefaultContext == null)
				return null;
			XComponentContext context = (XComponentContext) UnoRuntime
					.queryInterface(XComponentContext.class, oDefaultContext);
			return (XMultiServiceFactory) UnoRuntime.queryInterface(
					XMultiServiceFactory.class, context.getServiceManager());
		} catch (Exception e) {
			e.printStackTrace();
		} catch (java.lang.Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public void setFocus() {

	}

}
