/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */
package com.ibm.productivity.tools.samples.presentation.ui;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import com.ibm.productivity.tools.samples.presentation.Activator;
import com.ibm.productivity.tools.samples.presentation.util.PageUtil;
import com.ibm.productivity.tools.ui.RichDocumentViewFactory;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.ibm.productivity.tools.ui.views.RichDocumentViewSelection;
import com.sun.star.awt.Point;
import com.sun.star.awt.Size;
import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.drawing.XDrawPage;
import com.sun.star.drawing.XDrawPages;
import com.sun.star.drawing.XShape;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.uno.UnoRuntime;

public class SampleShelfView extends ViewPart {
	
	private static String file = "resource/Sample.odp";
	
	private XModel xModel;
	private Button insertButton;
	private Button removeButton;
	private Button copyButton;
	private CCombo numberCombo;
	
	public void createPartControl(Composite parent) {
		parent.setLayout(new GridLayout());
		createOpenGroup(parent);
		createInsertPageGroup(parent);
		createRemoveGroup(parent);
		createCopyGroup(parent);
		addSelectionListener();
		refreshView(xModel);
		
	}
	
	private void createOpenGroup(Composite parent){
		Group group = new Group(parent,SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_BOTH));
		group.setLayout(new GridLayout());
		Button loadButton = new Button(group,SWT.NONE);
		loadButton.setText("Open document");
		loadButton.setLayoutData(new GridData(GridData.BEGINNING,GridData.BEGINNING,false,false));
		loadButton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent arg0){
				RichDocumentView tempView = RichDocumentViewFactory.openView(Activator.getResolvedPath(file), false);
			}
		});
		Label label = new Label(group,SWT.WRAP);
		label.setLayoutData(new GridData(GridData.FILL_BOTH));
		label.setText("Click \"Open document\" button to open a sample presentation.");
	}

	private void createInsertPageGroup(Composite parent) {
		Group group = new Group(parent,SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_BOTH));
		group.setLayout(new GridLayout(2,false));
		Label label = new Label(group,SWT.NONE);
		label.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
		label.setText("title:");
		final Text titleText = new Text(group,SWT.BORDER|SWT.MULTI|SWT.WRAP|SWT.H_SCROLL|SWT.V_SCROLL);
		titleText.setLayoutData(new GridData(GridData.FILL,GridData.BEGINNING,false,false));
		
		Label contentLabel = new Label(group,SWT.NONE);
		contentLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
		contentLabel.setText("content:");
		final Text contentText = new Text(group,SWT.BORDER|SWT.MULTI|SWT.WRAP|SWT.H_SCROLL|SWT.V_SCROLL);
		contentText.setLayoutData(new GridData(GridData.FILL_BOTH));
	
		insertButton = new Button(group,SWT.NONE);
		insertButton.setLayoutData(new GridData(GridData.BEGINNING,GridData.BEGINNING,false,false));
		insertButton.setText("Insert after");
		insertButton.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent arg0) {
				int pageIndex = getPageIndex();
				String title = titleText.getText();
				if(title==null)
					title = "";
				String content = contentText.getText();
				if(content==null)
					content = "";
				insertPage(pageIndex,title,content);
				XDrawPages xDrawPages = PageUtil.getXDrawPages(xModel);
				int nPageCount = xDrawPages.getCount();
				refreshNumberCombo(nPageCount);
			}});
		
		numberCombo = new CCombo(group,SWT.BORDER);
		numberCombo.setLayoutData(new GridData(GridData.FILL,GridData.BEGINNING,false,false));
		
		Label descripeLabel = new Label(group,SWT.WRAP);
		GridData labelData = new GridData(GridData.FILL_BOTH);
		labelData.horizontalSpan = 2;
		descripeLabel.setLayoutData(labelData);
		descripeLabel.setText("Click \"Insert after\" button to insert a new page with the title and content above.Select the page number that you want to insert after.");
	}

	private void createRemoveGroup(Composite parent) {
		Group group = new Group(parent,SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_BOTH));
		group.setLayout(new GridLayout());

		removeButton = new Button(group,SWT.NONE);
		removeButton.setLayoutData(new GridData(GridData.BEGINNING,GridData.BEGINNING,false,false));
		removeButton.setText("Remove");
		removeButton.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent arg0) {
				PageUtil.removeCurrentPage(xModel);
				XDrawPages xDrawPages = PageUtil.getXDrawPages(xModel);
				int nPageCount = xDrawPages.getCount();
				refreshNumberCombo(nPageCount);
			}});
		
		Label label = new Label(group,SWT.WRAP);
		label.setLayoutData(new GridData(GridData.FILL_BOTH));
		label.setText("Click \"Remove\" button to remove current page.");
	}
	
	private void createCopyGroup(Composite parent) {
		Group group = new Group(parent,SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_BOTH));
		group.setLayout(new GridLayout());
		
		copyButton = new Button(group,SWT.NONE);
		copyButton.setLayoutData(new GridData(GridData.BEGINNING,GridData.BEGINNING,false,false));
		copyButton.setText("Copy");
		copyButton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent arg0) {
				PageUtil.copyCurrentPage(xModel);
				XDrawPages xDrawPages = PageUtil.getXDrawPages(xModel);
				int nPageCount = xDrawPages.getCount();
				refreshNumberCombo(nPageCount);
			}});
		
		Label label = new Label(group,SWT.WRAP);
		label.setLayoutData(new GridData(GridData.FILL_BOTH));
		label.setText("Click \"Copy\" button to copy current page,the new duplicated page will be inserted after current page.");
	}
	
	
	private void addSelectionListener(){
		ISelectionService service = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService();
		service.addSelectionListener(
				new ISelectionListener() {
					public void selectionChanged(IWorkbenchPart arg0, final ISelection arg1) {
						//it's better to call UNO API in a job
						Job job = new Job("refresh view"){
							protected IStatus run(IProgressMonitor arg0) {
								IAdaptable adaptable = (IAdaptable) arg1;
								RichDocumentViewSelection selection = (RichDocumentViewSelection)
							    adaptable.getAdapter(RichDocumentViewSelection.class);        
								RichDocumentView view = selection.getView();		
								Object unoModel = view.getUNOModel();
								xModel = (XModel)UnoRuntime.queryInterface(XModel.class, unoModel);
								refreshView(xModel);
								return Status.OK_STATUS;
							}
						};
						job.schedule();
						
					}
				});
	}
	
	/**
	 * refresh shelf view when documented changed
	 * @param xComponent 
	 */
	private void refreshView(XComponent xComponent){
		if (xComponent != null) {
			XServiceInfo info = (XServiceInfo) UnoRuntime.queryInterface(
					XServiceInfo.class, xComponent);
			if (info.supportsService("com.sun.star.presentation.PresentationDocument")){
				activeView(xComponent);
			} else 
				deactiveView();
			
		}else
			deactiveView();
	}
	
	/**
	 * refresh page number when presentation document changed.
	 * @param xComponent
	 */
	private void refreshNumberCombo(int nPageCount){
		numberCombo.removeAll();
		numberCombo.add("current page");
		for(int i=1;i<=nPageCount;i++){
			numberCombo.add(String.valueOf(i));
		}
		numberCombo.select(0);
	}
	
	/**
	 * enable button and combo if select a presentation document
	 */
	private void activeView(XComponent xComponent){
		XDrawPages xDrawPages = PageUtil.getXDrawPages(xComponent);
		final int nPageCount = xDrawPages.getCount();
		//this method is called in a job,so we have to use a runnable to run UI operation
		Display.getDefault().asyncExec(new Runnable(){
			public void run() {
				refreshNumberCombo(nPageCount);
				insertButton.setEnabled(true);
				removeButton.setEnabled(true);
				copyButton.setEnabled(true);
			}});
	}
	/**
	 * disable button and combo if select a non-presentation document
	 */
	private void deactiveView(){
		//this method is called in a job,so we have to use a runnable to run UI operation
		Display.getDefault().asyncExec(new Runnable(){
			public void run() {
				numberCombo.removeAll();
				insertButton.setEnabled(false);
				removeButton.setEnabled(false);
				copyButton.setEnabled(false);
			}});
	}
	
	/**
	 * get index of page to insert
	 * @return 
	 */
	private int getPageIndex(){
		int pageIndex = 0;
		if(numberCombo.getSelectionIndex()>0)
			pageIndex = Integer.parseInt(numberCombo.getText())-1;
		else
			pageIndex = getCurrentPageIndex();
		return pageIndex;
	}
	
	/**
	 * get index of current selected page 
	 * @return
	 */
	private int getCurrentPageIndex(){
		int pageIndex = 0;
		XDrawPage currentPage = PageUtil.getCurrentPage(xModel);
		XDrawPages xDrawPages = PageUtil.getXDrawPages(xModel);
		for(int i=0;i<xDrawPages.getCount();i++){
			XDrawPage page = PageUtil.getXDrawPage(xDrawPages,i);
			if(page.equals(currentPage)){
				pageIndex = i;
				break;
			}
		}
		return pageIndex;
	}
	
	
	/**
	 * insert a page with a title and content,and set its master page and then add two shapes for the page
	 * @param pageIndex
	 * @param title
	 * @param content
	 */
	private void insertPage(int pageIndex,String title,String content){
		XDrawPage xNewDrawPage = PageUtil.insertDrawPage(xModel,pageIndex);
		//set master page for the new draw page
		XDrawPage masterPage = PageUtil.getMasterPageByIndex(xModel,1);
		PageUtil.setMasterPage(xNewDrawPage, masterPage);
		//add title shape for the new draw page
		PageUtil.addShape(xModel, xNewDrawPage,
				new Point( 2058, 1743 ), new Size( 3507, 23912 ),
				"com.sun.star.presentation.TitleTextShape",title);
		//add OutlinerShape for the new draw page
		PageUtil.addShape(xModel, xNewDrawPage,
				new Point( 2058, 5838 ), new Size( 13230, 23912 ),
				"com.sun.star.presentation.OutlinerShape",content);
		
       //add a EllipseShape for the new draw page
		XShape shape = PageUtil.addShape(xModel, xNewDrawPage,
				new Point( 3000, 9000 ), new Size( 7000, 5000 ),
				"com.sun.star.drawing.EllipseShape","New EllipseShape");
		//add a RectangleShape for the new draw page
		shape = PageUtil.addShape( xModel,xNewDrawPage,
				new Point( 18000, 9000 ), new Size( 7000, 5000 ),
				"com.sun.star.drawing.RectangleShape","New RectangleShape");
		
		//set propertites for the RectangleShape
		XPropertySet xPropSet = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, shape);
		try {
			//corner
			xPropSet.setPropertyValue("CornerRadius", new Integer(1000));
			//shadow
			xPropSet.setPropertyValue("Shadow", new Boolean(true));
			xPropSet.setPropertyValue("ShadowXDistance", new Integer(250));
			xPropSet.setPropertyValue("ShadowYDistance", new Integer(250));
		} catch (UnknownPropertyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PropertyVetoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	

	public void setFocus() {

	}

}
