/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */
package com.ibm.productivity.tools.samples.presentation.util;

import com.sun.star.awt.Point;
import com.sun.star.awt.Size;
import com.sun.star.drawing.XDrawPage;
import com.sun.star.drawing.XDrawPageDuplicator;
import com.sun.star.drawing.XDrawPages;
import com.sun.star.drawing.XDrawPagesSupplier;
import com.sun.star.drawing.XDrawView;
import com.sun.star.drawing.XMasterPageTarget;
import com.sun.star.drawing.XMasterPagesSupplier;
import com.sun.star.drawing.XShape;
import com.sun.star.drawing.XShapes;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.text.XText;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;

public class PageUtil {

	/**
	 * get current draw page of selected presentation
	 * 
	 * @param xModel
	 * @return
	 */
	public static XDrawPage getCurrentPage(XModel xModel) {
		XDrawView xDrawView = (XDrawView) UnoRuntime.queryInterface(
				XDrawView.class, xModel.getCurrentController());
		XDrawPage xpage = xDrawView.getCurrentPage();
		return xpage;
	}

	/**
	 * get the draw pages container
	 * 
	 * @param xComponent
	 * @return
	 */
	public static XDrawPages getXDrawPages(XComponent xComponent) {
		XDrawPagesSupplier xDrawPagesSupplier = (XDrawPagesSupplier) UnoRuntime
				.queryInterface(XDrawPagesSupplier.class, xComponent);
		XDrawPages xDrawPages = xDrawPagesSupplier.getDrawPages();
		return xDrawPages;
	}

	/**
	 * get draw page by pageIndex
	 * 
	 * @param xDrawPages
	 * @param pageIndex
	 * @return
	 */
	public static XDrawPage getXDrawPage(XDrawPages xDrawPages, int pageIndex) {
		XDrawPage xDrawPage = null;
		try {
			Object obj = xDrawPages.getByIndex(pageIndex);
			xDrawPage = (XDrawPage) UnoRuntime.queryInterface(XDrawPage.class,obj);
		} catch (IndexOutOfBoundsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return xDrawPage;
	}

	/**
	 * remove current page
	 */
	public static void removeCurrentPage(XModel xModel) {
		XDrawPages xDrawPages = getXDrawPages(xModel);
		XDrawPage xDrawPage = getCurrentPage(xModel);
		if (xDrawPage != null)
			xDrawPages.remove(xDrawPage);
	}

	/**
	 * Duplicate current page
	 */
	public static void copyCurrentPage(XModel xModel) {
		XDrawPage xDrawPage = getCurrentPage(xModel);
		XDrawPageDuplicator duplicator = (XDrawPageDuplicator) UnoRuntime
				.queryInterface(XDrawPageDuplicator.class, xModel);
		duplicator.duplicate(xDrawPage);
	}

	
	/**
	 * get a master page by index
	 * @param xComponent
	 * @param index
	 * @return
	 */
	public static XDrawPage getMasterPageByIndex(XComponent xComponent,int index){
		XMasterPagesSupplier xMasterPagesSupplier = (XMasterPagesSupplier) UnoRuntime
		.queryInterface(XMasterPagesSupplier.class, xComponent);
		XDrawPages xMasterPages = xMasterPagesSupplier.getMasterPages();
		XDrawPage xDrawPage = null;
		try {
			Object obj = xMasterPages.getByIndex(index);
			xDrawPage = (XDrawPage) UnoRuntime.queryInterface(XDrawPage.class,obj);
		} catch (IndexOutOfBoundsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
		return xDrawPage;
	}

	/**
	 * set master page for a draw page
	 * 
	 * @param xNewDrawPage
	 * @param masterPage
	 */
	public static void setMasterPage(XDrawPage xDrawPage, XDrawPage masterPage) {
		// query for MasterPageTarget of draw page
		XMasterPageTarget xMasterPageTarget = (XMasterPageTarget) UnoRuntime
				.queryInterface(XMasterPageTarget.class, xDrawPage);
		// set master page
		xMasterPageTarget.setMasterPage(masterPage);
	}

	/**
	 * insert a draw page by pageIndex
	 * 
	 * @param xComponent
	 * @param pageIndex
	 * @return
	 */
	public static XDrawPage insertDrawPage(XComponent xComponent, int pageIndex) {
		XDrawPages xDrawPages = getXDrawPages(xComponent);
		XDrawPage xNewDrawPage = xDrawPages.insertNewByIndex(pageIndex);
		return xNewDrawPage;
	}

	/**
	 * add shape to a page
	 * 
	 * @param page
	 * @param pos  ,position of the shape
	 * @param size ,size of the shape
	 * @param shapeType ,service name for the shape
	 * @param text, text shown on the shape
	 */
	public static XShape addShape(XComponent xComponent, XDrawPage page,
			Point pos, Size size, String shapeType, String text) {
		XShape xShape = null;
		try {
			XShapes xShapes = (XShapes) UnoRuntime.queryInterface(
					XShapes.class, page);
			XMultiServiceFactory xFactory = (XMultiServiceFactory) UnoRuntime
					.queryInterface(XMultiServiceFactory.class, xComponent);
			Object xObj = xFactory.createInstance(shapeType);
			xShape = (XShape) UnoRuntime.queryInterface(XShape.class, xObj);
			xShape.setPosition(pos);
			xShape.setSize(size);
			// add the shape
			xShapes.add(xShape);
			// set text for the shape
			XText xText = (XText) UnoRuntime.queryInterface(XText.class, xShape);
			xText.setString(text);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return xShape;
	}

}
