/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.spreadsheet.ui;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import com.ibm.productivity.tools.samples.spreadsheet.provider.FileListViewContentProvider;
import com.ibm.productivity.tools.samples.spreadsheet.provider.FileListViewLabelProvider;
import com.ibm.productivity.tools.samples.spreadsheet.util.SimpleCell;
import com.ibm.productivity.tools.samples.spreadsheet.util.SpreadsheetUtil;
import com.ibm.productivity.tools.ui.RichDocumentViewFactory;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.ibm.productivity.tools.ui.views.RichDocumentViewSelection;
import com.sun.star.frame.XModel;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.sheet.XSpreadsheetDocument;
import com.sun.star.sheet.XSpreadsheets;
import com.sun.star.uno.UnoRuntime;

public class ShelfView extends ViewPart {
	
//	static for side shelf float in a new window
	static private RichDocumentView view;
	
	private Composite parent;
	
	private Group fileListGroup;
	
	private ListViewer fileListViewer;
	
	private Group cellsGroup;	
	
//	static for side shelf float in a new window
	static private Text valueText;
	
//	static for side shelf float in a new window
	static private Text intXText;
	
//	static for side shelf float in a new window
	static private Text intYText;	

	public void createPartControl(Composite aParent) {
		parent = aParent;
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		parent.setLayout(layout);
		
		drawFileListGroup();

		drawCellsGroup();
		  
		addListener();		
	}
	
	/**
	 * draw a group for the demo files list
	 */
	private void drawFileListGroup() {
		fileListGroup = new Group(parent, SWT.NULL);				
        
        fileListGroup.setText(Constant.ODS_FILE_LIST_GROUP_NAME);
        GridData data = new GridData(GridData.CENTER);
        data.grabExcessHorizontalSpace = true;
        fileListGroup.setLayoutData(data);
        
        GridLayout layout = new GridLayout();
		layout.numColumns = 1;
        fileListGroup.setLayout(layout);
        
        fileListViewer = new ListViewer(fileListGroup);
        List list = fileListViewer.getList();
        
        data = new GridData(GridData.CENTER);
        data.grabExcessHorizontalSpace = true;
        data.minimumHeight = 60;
        data.minimumWidth = 120;
        list.setLayoutData(data);
                        
        fileListViewer.setContentProvider(new FileListViewContentProvider());
        fileListViewer.setLabelProvider(new FileListViewLabelProvider());
        fileListViewer.setInput(this); 
        fileListViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {				
				handleDoubleClick();
			}
		});
        list.setVisible(true);
        list.setToolTipText(Constant.ODS_FILE_LIST_TOOLTIP);
	} 
		
	/**
	 * draw a group for operating the cells 
	 */
	private void drawCellsGroup() {
		cellsGroup = new Group(parent, SWT.NULL);
		
		cellsGroup.setText(Constant.CELLS_GROUP_NAME);
        GridData data = new GridData(GridData.CENTER | GridData.FILL_HORIZONTAL);
        data.grabExcessHorizontalSpace = true;
        data.verticalIndent = 20;
        cellsGroup.setLayoutData(data);
        
        GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		cellsGroup.setLayout(layout);						
		
		createSeter();
        
        createChartbutton();
        
        CreatDataPilotButton();                               
	}
	
	/**
	 * add the selection changed event listener
	 */
	private void addListener() {
		
		ISelectionService service = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService();
		service.addSelectionListener(
				new ISelectionListener() {
					public void selectionChanged(IWorkbenchPart arg0, final ISelection arg1) {
						//it's better to call UNO API in a job
						Job job = new Job("set cell value"){
							protected IStatus run(IProgressMonitor arg0) {
							    IAdaptable adaptable = (IAdaptable) arg1;
								getCellValueAndAddress(adaptable);		
								return Status.OK_STATUS;
							}
						};
						job.schedule();
					}
				});
	}		
	
	/**
	 * create the get and set function for a cell. 
	 */
	private void createSeter() {
		Label seterLable = new Label(cellsGroup, SWT.LEFT);		
		GridData data = new GridData(GridData.CENTER | GridData.FILL_BOTH);
        data.grabExcessHorizontalSpace = true;
        data.horizontalSpan = 2;        
        seterLable.setLayoutData(data);
        seterLable.setText(Constant.SET_LABEL_NAME);
        
        Label valueLabel = new Label(cellsGroup, SWT.RIGHT);
        data = new GridData(SWT.RIGHT);
        data.horizontalSpan = 1;        
        valueLabel.setLayoutData(data);
        valueLabel.setText(Constant.VALUE_LABEL_NAME);
        valueLabel.setToolTipText(Constant.VALUE_LABEL_TOOLTIP_TEXT);
        
        valueText = new Text(cellsGroup, SWT.BORDER | SWT.LEFT);
        data = new GridData();
        data.horizontalSpan = 1;
        data.widthHint = 150;
        data.horizontalIndent = -35;
        valueText.setLayoutData(data);
        valueText.setToolTipText(Constant.VALUE_LABEL_TOOLTIP_TEXT);
        
        Label intX = new Label(cellsGroup, SWT.RIGHT);
        data = new GridData(SWT.RIGHT);
        data.horizontalSpan = 1;
        intX.setLayoutData(data);
        intX.setText(Constant.X_LABEL_NAME);
        intX.setToolTipText(Constant.X_LABEL_TOOLTIP_TEXT);
        
        intXText = new Text(cellsGroup, SWT.BORDER | SWT.LEFT);
        data = new GridData();
        data.horizontalSpan = 1;
        data.widthHint = 150;
        data.horizontalIndent = -35;       
        intXText.setLayoutData(data);
        intXText.setToolTipText(Constant.X_LABEL_TOOLTIP_TEXT);
        
        Label intY = new Label(cellsGroup, SWT.RIGHT);
        data = new GridData(SWT.RIGHT);
        data.horizontalSpan = 1;
        intY.setLayoutData(data);
        intY.setText(Constant.Y_LABEL_NAME);
        intY.setToolTipText(Constant.Y_LABEL_TOOLTIP_TEXT);
        
        intYText = new Text(cellsGroup, SWT.BORDER | SWT.LEFT);
        data = new GridData();
        data.horizontalSpan = 1;
        data.widthHint = 150;
        data.horizontalIndent = -35;        
        intYText.setLayoutData(data);
        intYText.setToolTipText(Constant.Y_LABEL_TOOLTIP_TEXT);
        
        Button seterButton = new Button(cellsGroup, SWT.CENTER);
        data = new GridData(GridData.CENTER);
        data.horizontalSpan = 2;
        data.widthHint = 70;
        data.horizontalIndent = 5;
        seterButton.setLayoutData(data);
        seterButton.setText(Constant.SET_BUTTON_NAME);
        seterButton.setToolTipText(Constant.SET_BUTTON_TOOLTIP_TEXT);
        
        seterButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {				
			}

			public void widgetSelected(SelectionEvent arg0) {
				SimpleCell sCell = new SimpleCell();
				sCell.setValue(valueText.getText());
				try {
					int intX = sCell.convertTextToInt(intXText.getText());
					int intY = Integer.parseInt(intYText.getText());
					if (intX != -1 && intY > 0) {
						sCell.setX(intX);
						sCell.setY(intY);
						setCellValue(sCell);
					} else {
						throw new NumberFormatException("The Col x must between A to IV and " +
								"Row y must be positive integer");
					}															
				} catch(NumberFormatException e) {
					e.printStackTrace();
				}												
			}			
		});
        
		Label separator = new Label(cellsGroup, SWT.SEPARATOR | SWT.HORIZONTAL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
        data.horizontalSpan = 2;
        separator.setLayoutData(data);
        separator.setVisible(true);
	}
	
	/**
	 * create a button to create a simple chart for this demo sheet
	 */
	private void createChartbutton() {
		Button chartButton = new Button(cellsGroup, SWT.CENTER | SWT.PUSH);        
        chartButton.setText(Constant.CREAT_CHART_NAME);
        chartButton.setToolTipText(Constant.CHART_BUTTON_TOOLTIP_TEXT);
        
        GridData data = new GridData(GridData.CENTER);
        data.horizontalSpan = 1;
        data.widthHint = 70;
        data.horizontalIndent = 5;
        chartButton.setLayoutData(data);
        
        chartButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {				
			}

			public void widgetSelected(SelectionEvent arg0) {				
				createChart();						
			}			
		});
	}
	
	/**
	 * create a button to create a simple data pilot for this demo sheet
	 *
	 */
	private void CreatDataPilotButton() {
		Button dataPilotButton = new Button(cellsGroup, SWT.CENTER | SWT.PUSH);
        dataPilotButton.setText(Constant.CREAT_DATA_PILOT_NAME);
        dataPilotButton.setToolTipText(Constant.DATA_PILOT_BUTTON_TOOLTIP_TEXT);
        
        GridData data = new GridData(GridData.CENTER);
        data.horizontalSpan = 1;
        data.widthHint = 70;
        data.horizontalIndent = 10;
        dataPilotButton.setLayoutData(data);
        
        dataPilotButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {				
			}

			public void widgetSelected(SelectionEvent arg0) {
				createDataPilot();								                
			}			
		});
	}
	
	/**
	 * open the demo file when the list item was be double clicked.
	 */
	private void handleDoubleClick() {
		ISelection selection = fileListViewer.getSelection();
		Object obj = ((IStructuredSelection)selection).getFirstElement();

		String displayName = obj.toString();
		String url = SpreadsheetUtil.getDocumentURL( displayName );		
		RichDocumentView tempView = RichDocumentViewFactory.openView(url, false);
		
		//guard code for reopen file, make the view always hold the current active view reference.
		if(tempView != null) {
			view = tempView;
		}										
	}
	
	/**
	 * set the cell value by its coordinate x and y contain in sCell.
	 * @param sCell 	 
	 */
	private void setCellValue(SimpleCell sCell) {
		XSpreadsheets xSheets = getSpreadsheets();
		SpreadsheetUtil.setvalue(xSheets, sCell);							
	}
	
	/**
	 * get the selected cell's value and coordinate 
	 * @param adaptable a adapter for the selected cell.
	 */
	private void getCellValueAndAddress(IAdaptable adaptable){		
		RichDocumentViewSelection selection = (RichDocumentViewSelection)
		    adaptable.getAdapter(RichDocumentViewSelection.class);        				
		view = selection.getView();		
		XModel model = (XModel)UnoRuntime.queryInterface(XModel.class, view.getUNOModel() );
		
		if (model == null) {
			return;
		}
		XServiceInfo info = (XServiceInfo) UnoRuntime.queryInterface(
		XServiceInfo.class, model);
		if (!info.supportsService("com.sun.star.sheet.SpreadsheetDocument")){
			return;
		}
		
		final SimpleCell sCell = SpreadsheetUtil.getSimpleCellwithValueAndAddress(model);		
		//this method is called in a job,so we have to use a runnable to run UI operation
		Display.getDefault().asyncExec(new Runnable(){
			public void run() {
				setCellInfoToSideshelf(sCell);	
			}});
	
	}	
	
	/**
	 * create a chart for this demo spreadsheet.
	 */
	private void createChart(){
		XSpreadsheets xSheets = getSpreadsheets();
		SpreadsheetUtil.createChart(xSheets);		
	}
	
	/**
	 * create a data pilot for this demo spreadsheet.
	 */
	private void createDataPilot() {
		XSpreadsheets xSheets = getSpreadsheets();
		SpreadsheetUtil.createDataPilot(xSheets);
	}
	
	/**
	 * get the current sheets
	 * @return current sheets
	 */
	private XSpreadsheets getSpreadsheets() {
		XSpreadsheets xSheets = null;
		
		Object model = view.getUNOModel();
		try {
			XSpreadsheetDocument sheetDocument = (XSpreadsheetDocument)UnoRuntime.
			queryInterface(XSpreadsheetDocument.class, model );			
			xSheets = sheetDocument.getSheets();
		} catch (NullPointerException e) {
			xSheets = null;
		}		
		
		return xSheets;
	}
	
	/**
	 * set the side shelf's spreadsheet group by the cell. 
	 * @param cell a Simple cell
	 */
	private void setCellInfoToSideshelf(SimpleCell cell){
		if(cell.getValue() == null){
			return;
		}		
		setValueText(cell.getValue());
		setXText(cell.getX());
		setYText(Integer.toString(cell.getY()));
	}
	
	public void setValueText(String text) {
		valueText.setText(text);
	}
	
	public void setXText(String text) {
		intXText.setText(text);
		
	}
	
	public void setYText(String text) {
		intYText.setText(text);
	}
	
	public void setFocus() {
		fileListViewer.getControl().setFocus();
	} 

}
