/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.spreadsheet.util;

/**
 * a simple class for spreadsheet's cell to contain a cell's value and address infomation.
 *
 */
public class SimpleCell {
	/**
	 * the vaule of this cell.
	 */
	private String value;
	
	/**
	 * the x coordinate of this cell's address
	 */
	private int x;
	
	/**
	 * the y coordinate of this cell's address
	 */
	private int y;
	
	public void setValue(String value) {
		this.value = value;
	}
	
	public void setX(int x){
		this.x = x;
	}
	
	public void setY(int y){
		this.y = y;
	}
	
	public String getValue(){
		return value;
	}
	
	public int getIntX(){
		return x;
	}
	
	public String getX(){
		return convertNumberToAlphabetic(x);
	}
	
	public int getY(){
		return y;
	}
	
	/**
	 * convert col x to alphabetic as spreadsheet shows
	 * @param x the col coordinate
	 * @return the alphabetic relate to the col coordinate
	 */
	private String convertNumberToAlphabetic(int x) {
		int temp, residue;
		String result;
		residue = (x - 1) % 26;
		result = Character.toString((char)(residue + 'A'));
		temp = (x - 1) / 26;
		while(temp > 0){
			residue = (temp - 1) % 26;
			result = Character.toString((char)(residue + 'A')) + result;
			temp = (temp - 1) / 26;
		}				
		return result;		
	}
	
	/**
	 * convert the alphabetic to the relative col coordinate
	 * @param text the col address of a cell
	 * @return col coordinate
	 */
	public int convertTextToInt(String text){
		int colX = 0;
		text.toUpperCase();
		if(!text.matches("[A-H]?[A-Z]|I[A-V]")) {
			return colX = -1;
		} else {
			char[] array = text.toCharArray();			
			for(int i = array.length, j = 0; i > 0 ; i--, j++){
				colX += (array[i - 1] - 'A' + 1) * Math.pow(26, j);
			}
			return colX;
		}		
	}
}
