/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.spreadsheet.util;

import java.io.IOException;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;

import com.ibm.productivity.tools.samples.spreadsheet.Activator;
import com.sun.star.awt.Rectangle;
import com.sun.star.beans.XPropertySet;
import com.sun.star.chart.XChartDocument;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNameAccess;
import com.sun.star.document.XEmbeddedObjectSupplier;
import com.sun.star.frame.XController;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.sheet.DataPilotFieldOrientation;
import com.sun.star.sheet.GeneralFunction;
import com.sun.star.sheet.XCellAddressable;
import com.sun.star.sheet.XCellRangeAddressable;
import com.sun.star.sheet.XDataPilotDescriptor;
import com.sun.star.sheet.XDataPilotTables;
import com.sun.star.sheet.XDataPilotTablesSupplier;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.sheet.XSpreadsheets;
import com.sun.star.table.CellAddress;
import com.sun.star.table.CellRangeAddress;
import com.sun.star.table.XCell;
import com.sun.star.table.XCellRange;
import com.sun.star.table.XTableChart;
import com.sun.star.table.XTableCharts;
import com.sun.star.table.XTableChartsSupplier;
import com.sun.star.text.XText;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XInterface;
import com.sun.star.view.XSelectionSupplier;

public class SpreadsheetUtil {
	
	private static int chartnumber = 0;
	
	private static int datapilotnumber = 0;

	/**
	 * convert the demo file's name in list into this file's URL
	 * @param displayName this file's name
	 * @return this file's URL
	 */
	public static String getDocumentURL( String displayName ) {
		String url = "";		
		String res = "resource/" + displayName + ".ods";		
		url = getResolvedPath( res );
		
		return url;
	}
	
	/**
	 * get this demo file's path
	 * @param file this file mame with postfix
	 * @return
	 */
	public static String getResolvedPath(String file) {
		String resolvedPath = null;
		Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
		if (bundle != null) {
			java.net.URL bundleURL = bundle.getEntry( "/" );
			if (bundleURL != null) {
				try {
					resolvedPath = FileLocator.resolve(bundleURL).getFile();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		String ret = resolvedPath+file;
		
		if(System.getProperty("os.name").startsWith("Windows")) {
			ret = ret.substring(1);
			ret = ret.replace('/', '\\');
		}		

		return ret;
	}
	
	/**
	 * set value to a cell in xSheets[0].
	 * @param xSheets a xSheets to operate
	 * @param cell a Simple cell
	 */
	public static void setvalue(XSpreadsheets xSheets, SimpleCell cell){
		XSpreadsheet xSheet = null;
		try {
			XIndexAccess xSheetsIA =(XIndexAccess)UnoRuntime.queryInterface( 
					XIndexAccess.class, xSheets);
			//get the first sheet in the document
			xSheet = (XSpreadsheet) UnoRuntime.queryInterface(
					XSpreadsheet.class, xSheetsIA.getByIndex( 0 ));
			
			XCell xCell = null;	        
	        try {
	        	xCell = xSheet.getCellByPosition(cell.getIntX() - 1, cell.getY() - 1);
	        } catch (IndexOutOfBoundsException ex) {
	            System.out.println("Could not get Cell");
	        }
	        
	        try{
	        	double d = Double.parseDouble(cell.getValue());
	        	xCell.setValue(d);
	        } catch(NumberFormatException e) {
	        	xCell.setFormula(cell.getValue());
	        }	        	        	        	        
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * get a cell's value and address
	 * @param model this cell's model
	 * @return a simple cell
	 */
	public static SimpleCell getSimpleCellwithValueAndAddress(XModel model){
		SimpleCell sCell = new SimpleCell();
		
		XController xController = ( XController )UnoRuntime.queryInterface(
				XController.class, model.getCurrentController());		
		XSelectionSupplier xSupplier = ( XSelectionSupplier )UnoRuntime.queryInterface(
				XSelectionSupplier.class, xController );
		Object selection = xSupplier.getSelection();		
		XServiceInfo xInfo = (XServiceInfo) UnoRuntime.queryInterface(
				XServiceInfo.class, selection);
		
		if (xInfo.supportsService("com.sun.star.sheet.SheetCell")) {
			XText xText = (XText) UnoRuntime.queryInterface(XText.class, selection);																					
			if (xText != null) {
				sCell.setValue(xText.getString());
			}
			
			XCell xCell = (XCell)UnoRuntime.queryInterface(XCell.class, selection);
			XCellAddressable xCellAddr = (XCellAddressable)UnoRuntime.queryInterface(
					XCellAddressable.class, xCell);
			
			CellAddress cAddress = xCellAddr.getCellAddress();
			sCell.setX(cAddress.Column + 1);
			sCell.setY(cAddress.Row + 1);					
		}
		
		return sCell;
	}
	
	/**
	 * create a chart with specified position and range.
	 * @param xSheets a spreadsheet to operate
	 */
	public static void createChart(XSpreadsheets xSheets){
		XSpreadsheet xSheet = null;
		try {
			XIndexAccess xSheetsIA =(XIndexAccess)UnoRuntime.queryInterface(
					XIndexAccess.class, xSheets);
			//get the first sheet in the document
			xSheet = (XSpreadsheet) UnoRuntime.queryInterface(
					XSpreadsheet.class, xSheetsIA.getByIndex( 0 ));
			
			Rectangle rect = new Rectangle();
			rect.X = 350;
			rect.Y = 2500;
			rect.Width = 11000;
			rect.Height = 6000;
	        
	        XCellRange xCRange = (XCellRange)UnoRuntime.queryInterface(XCellRange.class, xSheet);
	        XCellRange myRange = xCRange.getCellRangeByName("A1:E4");
	        XCellRangeAddressable xCRangeAddr = (XCellRangeAddressable)UnoRuntime.queryInterface(
	            XCellRangeAddressable.class, myRange);
	        CellRangeAddress myAddr = xCRangeAddr.getRangeAddress();
	        
	        CellRangeAddress[] cRAddr = new CellRangeAddress[1];
	        cRAddr[0] = myAddr;
	        XTableChartsSupplier xTCSupp = (XTableChartsSupplier)UnoRuntime.queryInterface(
	            XTableChartsSupplier.class, xSheet);
	        
	        XTableChart xTChart = null;	        
	        
	        XTableCharts xTCharts = xTCSupp.getCharts();
	        chartnumber++;
	        xTCharts.addNewByName("Example" + chartnumber, rect, cRAddr, true, true);
	        
	        // get the diagramm and Change some of the properties	        
	        try {
	        	xTChart = (XTableChart) (UnoRuntime.queryInterface(
	                XTableChart.class, ((XNameAccess) UnoRuntime.queryInterface(
	                                        XNameAccess.class, xTCharts)).getByName(
	                                        		"Example" + chartnumber)));
	            
	            XEmbeddedObjectSupplier xEOSupp = (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
	                XEmbeddedObjectSupplier.class, xTChart);
	            XInterface xInt = xEOSupp.getEmbeddedObject();
	            XChartDocument xChartDoc = (XChartDocument) UnoRuntime.queryInterface(
	            		XChartDocument.class,xInt);
	            //Thread.sleep(200);
	            XPropertySet xPS = (XPropertySet)UnoRuntime.queryInterface(
	                XPropertySet.class, xChartDoc.getTitle() );
	            xPS.setPropertyValue("String","Chart Example");
	        } catch (Exception e){
	            System.out.println("Changin Properties failed " + e);
	        }
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/** 
	 * create a data pilot with specified position and range.
	 * @param xSheets a spreadsheet to operate
	 */
	public static void createDataPilot(XSpreadsheets xSheets){
		XSpreadsheet xSheet = null;
		try {
			XIndexAccess xSheetsIA =(XIndexAccess)UnoRuntime.queryInterface( 
					com.sun.star.container.XIndexAccess.class, xSheets);
			// get the first sheet in the document
			xSheet = (XSpreadsheet) UnoRuntime.queryInterface(
					XSpreadsheet.class, xSheetsIA.getByIndex(0));
           // create a new DataPilot table
		    XDataPilotTablesSupplier xDPSupp = (XDataPilotTablesSupplier)UnoRuntime.queryInterface(
		    		XDataPilotTablesSupplier.class, xSheet);
		    XDataPilotTables xDPTables = xDPSupp.getDataPilotTables();
		    XDataPilotDescriptor xDPDesc = xDPTables.createDataPilotDescriptor();

		    // set source range (use data range from CellRange test)
		    CellRangeAddress aSourceAddress = createCellRangeAddress(xSheet, "A1:E4");
		    xDPDesc.setSourceRange(aSourceAddress);

		    // settings for fields
		    XIndexAccess xFields = xDPDesc.getDataPilotFields();
		    Object aFieldObj;
		    XPropertySet xFieldProp;

		    // use first column as column field
		    aFieldObj = xFields.getByIndex(0);
		    xFieldProp = (XPropertySet)UnoRuntime.queryInterface(
		    		XPropertySet.class, aFieldObj);
		    xFieldProp.setPropertyValue("Orientation", DataPilotFieldOrientation.COLUMN);

		    // use second column as data field, calculating the sum
		    aFieldObj = xFields.getByIndex(1);
		    xFieldProp = (XPropertySet)
		        UnoRuntime.queryInterface(XPropertySet.class, aFieldObj);
		    xFieldProp.setPropertyValue("Orientation", DataPilotFieldOrientation.DATA);
		    xFieldProp.setPropertyValue("Function", GeneralFunction.SUM);
		    
		    aFieldObj = xFields.getByIndex(2);
	        xFieldProp = (XPropertySet)UnoRuntime.queryInterface( 
	        		XPropertySet.class, aFieldObj );
	        xFieldProp.setPropertyValue( "Orientation", DataPilotFieldOrientation.DATA );
	        xFieldProp.setPropertyValue( "Function", GeneralFunction.AVERAGE );
	        
	        aFieldObj = xFields.getByIndex(3);
	        xFieldProp = (XPropertySet)UnoRuntime.queryInterface(
	        		XPropertySet.class, aFieldObj );
	        xFieldProp.setPropertyValue( "Orientation", DataPilotFieldOrientation.DATA );
	        xFieldProp.setPropertyValue( "Function", GeneralFunction.MAX );
	        
	        aFieldObj = xFields.getByIndex(4);
	        xFieldProp = (XPropertySet)
	            UnoRuntime.queryInterface( XPropertySet.class, aFieldObj );
	        xFieldProp.setPropertyValue( "Orientation", DataPilotFieldOrientation.DATA );
	        xFieldProp.setPropertyValue( "Function", GeneralFunction.MIN );

		    // select output position	        
	        datapilotnumber++;
		    CellAddress aDestAddress = createCellAddress(xSheet, "A20");		    
		    xDPTables.insertNewByName("DataPilotExample" + datapilotnumber, aDestAddress, xDPDesc);
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/** Creates a com.sun.star.table.CellAddress and initializes it
	  * with the given range.
	  * @param xSheet  The XSpreadsheet interface of the spreadsheet.
	  * @param aCell  The address of the cell (or a named cell).
	  */
	private static CellAddress createCellAddress(XSpreadsheet xSheet, String aCell ) {
			XCellAddressable xAddr = null;
			try {
				xAddr = (XCellAddressable)
				UnoRuntime.queryInterface(XCellAddressable.class,
						xSheet.getCellRangeByName(aCell).getCellByPosition(0, 0));
			} catch (IndexOutOfBoundsException e) {
				e.printStackTrace();
			}

			return xAddr.getCellAddress();

		}
	
	/** Creates a com.sun.star.table.CellRangeAddress and initializes
     *  it with the given range.
     *  @param xSheet  The XSpreadsheet interface of the spreadsheet.
     *  @param aRange  The address of the cell range (or a named range).
	 */
	private static CellRangeAddress createCellRangeAddress(XSpreadsheet xSheet, String aRange) {
		XCellRangeAddressable xAddr = (XCellRangeAddressable)
		UnoRuntime.queryInterface(XCellRangeAddressable.class, xSheet.getCellRangeByName(aRange));

		return xAddr.getRangeAddress();
	}
}
