/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.views;

import java.util.ArrayList;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchWindow;

import com.ibm.productivity.tools.samples.views.actions.DropDownAction;
import com.ibm.productivity.tools.samples.views.actions.SampleActionFactory;
import com.ibm.productivity.tools.ui.views.DefaultRichDocumentView;
import com.ibm.productivity.tools.ui.views.RichDocumentType;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.ibm.productivity.tools.ui.views.operations.LoadOperation;
import com.ibm.productivity.tools.ui.views.operations.NewOperation;
import com.ibm.productivity.tools.ui.views.operations.Operation;
import com.ibm.productivity.tools.ui.views.operations.OperationFactory;
import com.ibm.productivity.tools.ui.views.operations.OperationListener;
import com.ibm.rcp.jface.action.SActionContributionItem;
import com.ibm.rcp.jface.action.SToolBarManager;

/**
 * demo writer view which illustrates using of the DocumentView and 
 * adding toolbar and menubar to the view
 * 
 */
public class WriterView extends DefaultRichDocumentView {

	public static final String VIEW_ID = "com.ibm.productivity.tools.samples.views.WriterView";
	private static Object model;

	public WriterView() {
		super();
	}

	public void createPartControl(Composite parent) {	
		// must call super to create part control
		super.createPartControl(parent);
				
		// fill toolbar and menubar of this view
		fillToolbar();
		createMenubar();
		
		//add listener to udpate model
		OperationListener listener = new OperationListener() {

			public void afterExecute(Operation operation, RichDocumentView view) {
				if(operation instanceof LoadOperation) {
					model = ((LoadOperation)operation).getUNOModel();
				}
				else if(operation instanceof NewOperation) {
					model = ((NewOperation)operation).getUNOModel();
				}
			}

			public void beforeExecute(Operation operation, RichDocumentView view) {
				
			}
			
		};
		this.addOperationListener(listener);
		
		NewOperation operation = OperationFactory.createNewOperation(RichDocumentType.DOCUMENT_TYPE );
		this.executeOperation( operation );		
	}

	/**
	 * fill the toolbar of this view
	 */
	public void fillToolbar() {
		// get the toolbar manager
		SToolBarManager toolBarMgr = (SToolBarManager) getViewSite()
				.getActionBars().getToolBarManager();

		IWorkbenchWindow window = getSite().getWorkbenchWindow();

		// add a demo dropdown action
		toolBarMgr.add(createDropDownActionItem());

		// add two normal actions
		toolBarMgr.add(createActionContributionItem(window, "load"));
		toolBarMgr.add(new SActionContributionItem(SampleActionFactory.CLOSE.create(window)));
		
		toolBarMgr.add(new SActionContributionItem(SampleActionFactory.DEMO1.create(window)));
		toolBarMgr.add(new SActionContributionItem(SampleActionFactory.DEMO2.create(window)));		
	}

	/**
	 * create a menubar of the view
	 */
	public void createMenubar() {
		// get the menu manager
		IMenuManager menuMgr = getViewSite().getActionBars().getMenuManager();

		// add two actions
		IWorkbenchWindow window = getSite().getWorkbenchWindow();
		menuMgr.add(createActionContributionItem(window, "load"));
		menuMgr.add(createActionContributionItem(window, "save"));

	}

	/**
	 * create an action contribution item from the specified action id.
	 * 
	 * @param window
	 *            the workbench window
	 * @return the created action contribution item
	 */
	private SActionContributionItem createActionContributionItem(
			IWorkbenchWindow window, String actionId) {
		SActionContributionItem item = new SActionContributionItem(
				SampleActionFactory.getFactory(actionId).create(window));
		return item;
	}

	/**
	 * create the demo dropdown action contribution item.
	 * 
	 * @return the created dropdown action contribution item
	 */
	private SActionContributionItem createDropDownActionItem() {
		//create an array and fill it with the items of the dropdown action contribution item
		ArrayList menuItemColl = new ArrayList();
		IWorkbenchWindow window = getSite().getWorkbenchWindow();

		menuItemColl.add(createActionContributionItem(window, "load"));
		menuItemColl.add(createActionContributionItem(window, "save"));

		//create the dropdown action
		IAction dropDownAction = new DropDownAction(menuItemColl);
		dropDownAction.setText("Open");
		dropDownAction.setId("DropDownTest");
		dropDownAction.setImageDescriptor(Activator
				.getImageDescriptor("resources/IDocInteligente.png"));
		dropDownAction.setDisabledImageDescriptor(Activator
				.getImageDescriptor("resources/IDocInteligente.png"));
		dropDownAction.setHoverImageDescriptor(Activator
				.getImageDescriptor("resources/IDocInteligente.png"));
		dropDownAction.setToolTipText("Test Drop Down");

		//finally create the dropdown action contribution item
		SActionContributionItem item = new SActionContributionItem(
				dropDownAction);
		
		return item;

	}

	public static Object getCurrentUNOModel() {
		return model;
	}

}
