/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.views.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import com.ibm.productivity.tools.samples.views.WriterView;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.text.XRelativeTextContentInsert;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextTable;
import com.sun.star.text.XTextTablesSupplier;
import com.sun.star.uno.UnoRuntime;

/**
 * This demo action replaces the first table of the current document
 *
 */
public class Demo1Action extends Action implements IWorkbenchAction {

	public void run() {
		// get the document service
		XTextDocument document = (XTextDocument) UnoRuntime.queryInterface(
				XTextDocument.class, WriterView.getCurrentUNOModel());
		// query the service factory for the document
		XMultiServiceFactory factory = (XMultiServiceFactory) UnoRuntime
				.queryInterface(XMultiServiceFactory.class, document);
		// query the table supplier
		XTextTablesSupplier supplier = (XTextTablesSupplier) UnoRuntime
				.queryInterface(XTextTablesSupplier.class, document);
		// get the names of all tables in the document
		String[] names = supplier.getTextTables().getElementNames();
		if(names.length==0) return;
		System.out.println("The first table:" + names[0]);
		try {
			// get the first table, we suppose there are at list one table in
			// the document here
			XTextTable table = (XTextTable) UnoRuntime.queryInterface(
					XTextTable.class, supplier.getTextTables().getByName(
							names[0]));
			// create a new table with 10 columns and 10 rows
			XTextTable table2 = (XTextTable) UnoRuntime.queryInterface(
					XTextTable.class, factory
							.createInstance("com.sun.star.text.TextTable"));
			table2.initialize(10, 10);
			// create a new Paragraph
			XTextContent xNewPara = (XTextContent) UnoRuntime.queryInterface(
					XTextContent.class, factory
							.createInstance("com.sun.star.text.Paragraph"));
			XRelativeTextContentInsert inserter = (XRelativeTextContentInsert) UnoRuntime
					.queryInterface(XRelativeTextContentInsert.class, document
							.getText());
			// Insert the new Paragraph after the found table
			inserter.insertTextContentAfter(xNewPara, table);
			// Insert the new Table after the new Paragraph
			document.getText().insertTextContent(
					xNewPara.getAnchor().getStart(), table2, false);
			// dispose the found table
			table.dispose();
		} catch (NoSuchElementException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void dispose() {
		
	}
}
