/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7R6EE5                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2009  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.views.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import com.ibm.rcp.jface.action.SActionContributionItem;

/**
 * Action for dropdown menu
 * 
 */
public class DropDownAction extends Action implements IMenuCreator,
		IWorkbenchAction {

	private Menu _menu;// stores the menu created by this dropdown action
	private ArrayList menuItems;// the action contribution items for the
	
	public DropDownAction() {
		super();

		// set this action's menu creator to itself to create drop down menu
		setMenuCreator(this);
	}

	/**
	 * Constructor with collection of contribution items
	 * 
	 * @param contributionItems
	 *            This contributionItems are expected to be implementations of
	 *            <code>IContributionItem</code>, normally
	 *            <code>ActionCOntributionItems</code>
	 */
	public DropDownAction(ArrayList contributionItems) {
		super();
		
		// set this action's menu creator to itself to create drop down menu
		setMenuCreator(this);
		
		this.menuItems = contributionItems;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IMenuCreator#dispose()
	 */
	public void dispose() {
		if(_menu != null)
			_menu.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Control)
	 */
	public Menu getMenu(Control parent) {
		if (_menu == null)
			_menu = new Menu(parent);

		// fill the dropdown menu
		for (Iterator iter = menuItems.iterator(); iter.hasNext();) {
			IContributionItem menuItem = (IContributionItem) iter.next();
			menuItem.fill(_menu, -1);
		}
		return _menu;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Menu)
	 */
	public Menu getMenu(Menu menu) {
		// fill the dropdown menu
		for (Iterator iter = menuItems.iterator(); iter.hasNext();) {
			IContributionItem menuItem = (IContributionItem) iter.next();
			menuItem.fill(menu, -1);
		}
		return menu;
	}

	/**
	 * @return the item of dropdown menu
	 */
	public Collection getMenuItems() {
		return menuItems;
	}

	/**
	 * @param menuItems the menu items of the dropdown menu
	 * 
	 */
	public void setMenuItems(ArrayList menuItems) {
		this.menuItems = menuItems;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		//run the new action
		((SActionContributionItem)menuItems.get(0)).getAction().run();
	}

}
